package com.agilecontrol.nea.core.web.shell;

import argparser.ArgParser;
import argparser.BooleanHolder;
import argparser.IntHolder;
import com.agilecontrol.csvreader.CsvReader;
import com.agilecontrol.nea.core.control.web.UserWebImpl;
import com.agilecontrol.nea.core.control.web.WebUtils;
import com.agilecontrol.nea.core.query.QueryEngine;
import com.agilecontrol.nea.core.query.QueryUtils;
import com.agilecontrol.nea.core.report.ReportUtils;
import com.agilecontrol.nea.core.web.shell.AbstractShellCmd;
import com.agilecontrol.nea.core.web.shell.Shell;
import com.agilecontrol.nea.util.StringUtils;
import com.agilecontrol.nea.util.Tools;
import com.agilecontrol.nea.util.Validator;
import java.io.File;
import java.nio.charset.Charset;
import java.sql.Connection;
import java.sql.ResultSet;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.servlet.http.HttpServletRequest;
import org.json.JSONObject;

@Shell(
   alias = "sql,select",
   admin = true,
   help = "sql [-a] [-n lines] SQL 直接执行SQL语句,生成文本文件。默认显示10行，-a 表示显示所有行, -n 指定行数。也可以忽略sql前缀直接写select语句"
)
public class ExecSQL extends AbstractShellCmd {

   private static final int DISPLAY_LINES = 10;


   protected JSONObject process(String args, JSONObject envObj) throws Exception {
      JSONObject jo = null;
      jo = new JSONObject();
      long startTime = System.currentTimeMillis();
      BooleanHolder allHolder = new BooleanHolder();
      IntHolder linesHolder = new IntHolder();
      String cmdline = envObj.getString("cmdline");
      String sql;
      if(cmdline.startsWith("sql")) {
         String[] displayLines = args.split(" ");
         ArgParser request = new ArgParser("");
         request.addOption("-a %v #all", allHolder);
         request.addOption("-n %d #lines", linesHolder);
         String ru = null;
         String[] name = request.matchAllArgs(displayLines, 0, 0);
         if(name != null) {
            ru = name[0];

            for(int svrPath = 1; svrPath < name.length; ++svrPath) {
               ru = ru + " " + name[svrPath];
            }
         }

         sql = ru.trim();
      } else {
         sql = cmdline;
      }

      if(sql.endsWith(";")) {
         sql = sql.substring(0, sql.length() - 1);
      }

      if(Validator.isNull(sql)) {
         jo.put("help", this.getHelp(((UserWebImpl)envObj.get("userweb")).getLocale()));
         jo.put("message", "sql 语句为空");
         jo.put("code", -1);
         return jo;
      } else {
         int var41 = allHolder.value?Integer.MAX_VALUE:(linesHolder.value > 0?linesHolder.value:10);
         HttpServletRequest var42 = (HttpServletRequest)envObj.get("request");
         ReportUtils var43 = new ReportUtils(var42);
         String var44 = var43.getUserName();
         String var45 = var43.getExportRootPath() + File.separator + var43.getUser().getClientDomain() + File.separator + this.userWeb.getUserId();
         SimpleDateFormat sdf = new SimpleDateFormat("MMddHHmm");
         String fileName = "sql" + sdf.format(new Date()) + ".csv";
         String filePath = var45 + File.separator + fileName;
         File file = new File(filePath);
         File svrDir = new File(var45);
         if(!svrDir.isDirectory()) {
            svrDir.mkdirs();
         }

         Connection conn = null;
         Object rs = null;
         boolean count = false;
         boolean isSelect = false;

         int var46;
         label196: {
            JSONObject sb;
            try {
               conn = QueryEngine.getInstance().getConnection();
               if(sql.toLowerCase().startsWith("select")) {
                  isSelect = true;
                  var46 = QueryUtils.queryToFile(sql, filePath, conn);
               } else {
                  conn.setAutoCommit(true);
                  var46 = conn.createStatement().executeUpdate(sql);
               }
               break label196;
            } catch (Throwable var39) {
               this.logger.error("Fail to execute " + args, var39);
               jo.put("code", 0);
               jo.put("message", var39.getMessage());
               sb = jo;
            } finally {
               try {
                  if(rs != null) {
                     ((ResultSet)rs).close();
                  }
               } catch (Throwable var38) {
                  ;
               }

               try {
                  if(conn != null) {
                     conn.close();
                  }
               } catch (Throwable var37) {
                  ;
               }

            }

            return sb;
         }

         String duration = ((DecimalFormat)QueryUtils.floatFormatter.get()).format((double)(System.currentTimeMillis() - startTime) / 1000.0D);
         jo.put("code", 0);
         StringBuilder var47 = new StringBuilder("Schema " + this.userWeb.getDataSource() + ", duration " + duration + " seconds, count ");
         var47.append(var46);
         if(isSelect) {
            var47.append(", size ").append(Tools.formatSize(file.length())).append(", first ").append(var41).append(" lines:<br/>").append(var41 != 1 && var46 != 1?this.csvToHtmlTable(filePath, var41):this.csvToObjTable(filePath));
            if(var46 > 0) {
               jo.put("fileurl", WebUtils.getServletContext().getContextPath() + "/servlets/binserv/Download?filename=" + fileName);
            } else {
               file.delete();
            }
         }

         jo.put("message", var47.toString());
         return jo;
      }
   }

   protected String csvToObjTable(String file) throws Exception {
      CsvReader reader = new CsvReader(file, ',', Charset.forName("UTF-8"));
      reader.readHeaders();
      String[] headers = reader.getHeaders();
      String[] values = null;
      if(reader.readRecord()) {
         values = reader.getValues();
      }

      reader.close();
      StringBuilder sb = new StringBuilder();
      sb.append("<table class=\'shell_result_sql_object\'>");

      for(int i = 0; i < headers.length; ++i) {
         String header = headers[i];
         String value = values == null?"":values[i];
         sb.append("<tr><td nowrap>").append(StringUtils.escapeHTMLTags(header, true)).append("</td>").append("<td>").append(StringUtils.escapeHTMLTags(value, true)).append("</td></tr>");
      }

      sb.append("</table>");
      reader.close();
      return sb.toString();
   }

   protected String csvToHtmlTable(String file, int lines) throws Exception {
      CsvReader reader = new CsvReader(file, ',', Charset.forName("UTF-8"));
      reader.readHeaders();
      String[] headers = reader.getHeaders();
      StringBuilder sb = new StringBuilder();
      sb.append("<table class=\'shell_result_sql_table\'><thead><tr>");
      String[] cnt = headers;
      int values = headers.length;

      for(int var8 = 0; var8 < values; ++var8) {
         String header = cnt[var8];
         sb.append("<td nowrap>").append(StringUtils.escapeHTMLTags(header, true)).append("</td>");
      }

      sb.append("</tr></thead>");
      int var12 = 0;
      if(lines < 0) {
         lines = Integer.MAX_VALUE;
      }

      while(var12 < lines && reader.readRecord()) {
         String[] var13 = reader.getValues();
         sb.append("<tr>");
         String[] var14 = var13;
         int var15 = var13.length;

         for(int var10 = 0; var10 < var15; ++var10) {
            String v = var14[var10];
            sb.append("<td nowrap>").append(StringUtils.escapeHTMLTags(v, true)).append("</td>");
         }

         sb.append("</tr>");
         ++var12;
      }

      sb.append("</table>");
      reader.close();
      return sb.toString();
   }
}
